/**
 * 
 */
package org.springblade.csetl.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.log.annotation.ApiLog;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.csetl.entity.CsDsyJob;
import org.springblade.csetl.entity.CsDsyJobCategory;
import org.springblade.csetl.service.CsDsyJobCategoryService;
import org.springblade.csetl.service.CsDsyJobService;
import org.springblade.system.client.DataPermClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.List;

/**
 *	数据同步任务分类 操作处理
 *
 * @author aokunsang
 * @date   2023-09-21
 */
@RestController
@RequestMapping("/cs/dsy-job-category")
@Api(value = "数据同步任务分类相关接口", tags = "数据同步任务分类接口")
public class CsDsyJobCategoryController extends BladeController {

	@Autowired
	private CsDsyJobCategoryService dsyJobCategoryService;
	
	@Autowired
	private CsDsyJobService dsyJobService;
	
	@Autowired
	private DataPermClient dataPermClient;
	
	/**
	 * 	列表数据
	 * @param dsyJobCategory
	 * @return
	 */
	@GetMapping("/list")
	@ApiOperation(value = "查询数据同步任务分类列表", httpMethod = "GET")
	public R<List<CsDsyJobCategory>> list(CsDsyJobCategory dsyJobCategory){
		QueryWrapper<CsDsyJobCategory> queryWrapper = Condition.getQueryWrapper(dsyJobCategory);
		long currUserId = getUser().getUserId();
		List<CsDsyJobCategory> dataList = dsyJobCategoryService.list(queryWrapper.lambda().orderByDesc(CsDsyJobCategory::getSortNum).orderByDesc(CsDsyJobCategory::getId));
		Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(currUserId);
		if(checkSuperPermRet != null && checkSuperPermRet) { //是否有超级权限
			return R.data(dataList);
		}else {
			//查询所有任务有权限的分类ID集合
			List<Integer> categoryIdList = dsyJobService.categoryIdListOfPerm(currUserId);
			//1.获取这些分类的父级ID[包含自己创建的分类父级别ID]
			dataList.stream().filter(f -> (categoryIdList.contains(f.getId()) || f.getCreateUserId().longValue() == currUserId) && f.getParentId() != 0).map(CsDsyJobCategory::getParentId).forEach(categoryIdList::add);
			//2.删除不在权限内的分类[且createUserId不是本人的]
			dataList.removeIf(y -> !categoryIdList.contains(y.getId()) && y.getCreateUserId().longValue() != currUserId);
			return R.data(dataList);
		}
	}
	
	/**
	 *	修改数据接口
	 * @param dsyJobCategory
	 * @return
	 */
	@ApiLog("数据同步任务分类")
	@PostMapping("/submit")
	@ApiOperation(value = "增加或修改数据同步任务分类", httpMethod = "POST")
	public R<String> doUpdate(@Validated @RequestBody CsDsyJobCategory dsyJobCategory) {
		if(dsyJobCategory.getId() == null) {
			dsyJobCategory.setCreateTime(LocalDateTime.now());
			dsyJobCategory.setCreateUserId(AuthUtil.getUserId());
			dsyJobCategory.setCreateBy(AuthUtil.getNickName());
		}
		return R.status(dsyJobCategoryService.saveOrUpdate(dsyJobCategory));
	}
	
	/**
	 * 	删除数据接口
	 * @param id
	 * @return
	 */
	@ApiLog("数据同步任务分类")
	@PostMapping("/delete")
	@ApiOperation(value = "根据ID删除数据同步任务分类", httpMethod = "POST")
	public R<String> doDelete(@RequestParam Integer id) {
		long cnt = dsyJobService.count(Wrappers.<CsDsyJob>lambdaQuery().eq(CsDsyJob::getCategoryId, id));
		if(cnt > 0) {
			return R.fail("该分类下有任务，不能删除。");
		}
		long count = dsyJobCategoryService.count(Wrappers.<CsDsyJobCategory>lambdaQuery().eq(CsDsyJobCategory::getParentId, id));
		if(count > 0) {
			return R.fail("该分类下有子分类，不能删除。");
		}
		return R.status(dsyJobCategoryService.removeById(id));
	}
}