/**
 *
 */
package org.springblade.csetl.controller;

import cn.hutool.core.util.StrUtil;
import cn.hutool.cron.pattern.parser.PatternParser;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xxl.job.core.biz.AdminBiz;
import com.xxl.job.core.biz.model.JobInfo;
import com.xxl.job.core.biz.model.ReturnT;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.log.annotation.ApiLog;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.csetl.dto.CsDsyJobDto;
import org.springblade.csetl.entity.CsDsyJob;
import org.springblade.csetl.entity.CsDsyJobInstance;
import org.springblade.csetl.enums.CsInstanceRunType;
import org.springblade.csetl.enums.CsJobStatus;
import org.springblade.csetl.service.CsDsyJobInstanceService;
import org.springblade.csetl.service.CsDsyJobService;
import org.springblade.etl.enums.InstanceStatus;
import org.springblade.system.client.DataPermClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.Optional;

/**
 *	数据同步任务 操作处理
 *
 * @author aokunsang
 * @date 2023-09-21
 */
@Slf4j
@RestController
@RequestMapping("/cs/dsy-job")
@Api(value = "数据同步任务相关接口", tags = "数据同步任务接口")
public class CsDsyJobController extends BladeController {

    @Autowired
    private CsDsyJobService dsyJobService;

    @Autowired
    private CsDsyJobInstanceService dsyJobInstanceService;

    @Autowired
    private AdminBiz adminBiz;

    @Autowired
    private DataPermClient dataPermClient;

    @Value("${xxl.job.executor.groupid}")
    private int groupId;

    /**
     * 	列表数据
     * @param dsyJob
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询数据同步任务", httpMethod = "GET")
    public R<IPage<CsDsyJob>> pageList(CsDsyJob dsyJob, Query query) {
        Long userId = getUser().getUserId();
        LambdaQueryWrapper<CsDsyJob> queryWrapper = Condition.getQueryWrapper(dsyJob).lambda().orderByDesc(CsDsyJob::getId);
        Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(getUser().getUserId());
        if (checkSuperPermRet == null || !checkSuperPermRet) {
            queryWrapper.and(t -> {
                t.eq(CsDsyJob::getCreateUserId, userId)
                        .or().inSql(CsDsyJob::getId, "select a.job_id from cs_dsy_job_permission a inner join data_permission_user b on a.data_perm_id = b.data_perm_id where b.user_id = " + userId)
                        .or().eq(CsDsyJob::getPublicStatus, 1);
            });
        }
        return R.data(dsyJobService.page(Condition.getPage(query), queryWrapper));
    }

    /**
     * 单条数据查询接口
     * @param id
     * @return
     */
    @GetMapping("/info")
    @ApiOperation(value = "根据ID获取数据同步任务", httpMethod = "GET")
    public R<CsDsyJobDto> info(@RequestParam Integer id) {
        CsDsyJob dsyJob = dsyJobService.getById(id);
        CsDsyJobDto dsyJobDto = new CsDsyJobDto();
        dsyJobDto.setJob(dsyJob);
        return R.data(dsyJobDto);
    }

    /**
     *	修改数据接口
     * @param dsyJobDto
     * @return
     */
    @ApiLog("数据同步任务")
    @PostMapping("/submit")
    @ApiOperation(value = "增加或修改数据同步任务", httpMethod = "POST")
    public R<Integer> doUpdate(@Validated @RequestBody CsDsyJobDto dsyJobDto) {
        CsJobStatus jobStatus = dsyJobDto.getJob().getJobStatus();
        Integer jobId = dsyJobService.doSaveOrUpdate(dsyJobDto);
        String suffixMsg = "", errMsg = "";
        if (jobStatus == CsJobStatus.ONLINE_RUN) { //保存并运行
            try {
                dsyJobService.jobRunLogic(jobId, CsInstanceRunType.MANUAL);
                suffixMsg = ",并且已提交执行";
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                errMsg = e.getMessage();
                suffixMsg = ",但是提交执行失败";
            }
        }
        R<Integer> ret = R.data(jobId, "任务保存成功" + suffixMsg);
        ret.setExt(errMsg);
        return ret;
    }

    /**
     * 运行任务
     * @param jobId
     * @return
     */
    @PostMapping("/run/{jobId}")
    @ApiOperation(value = "运行任务", httpMethod = "POST")
    public R<Boolean> run(@PathVariable Integer jobId) {
        dsyJobService.jobRunLogic(jobId, CsInstanceRunType.MANUAL);
        return R.success("任务已提交执行");
    }

    /**
     * 	删除数据接口
     * @param id
     * @return
     */
    @PostMapping("/delete")
    @ApiOperation(value = "根据ID删除数据同步任务", httpMethod = "POST")
    public R<String> doDelete(@RequestParam Integer id) {
        CsDsyJob dbJob = dsyJobService.getById(id);
        if (dbJob.getJobStatus() == CsJobStatus.ONLINE) {
            return R.fail("任务需要先下线后才能删除");
        }
        return R.status(dsyJobService.removeById(id));
    }

    /**
     *
     * @param jobId
     * @param jobStatus
     * @return
     */
    @ApiLog("数据同步任务")
    @PostMapping("/switch/{jobId}")
    @ApiOperation(value = "切换任务状态", httpMethod = "POST")
    public R<String> switchStatus(@PathVariable Integer jobId, @RequestParam CsJobStatus jobStatus) {
        if (jobStatus != CsJobStatus.ONLINE && jobStatus != CsJobStatus.OFFLINE) {
            return R.fail("任务状态不正确");
        }
        if (jobStatus == CsJobStatus.OFFLINE && dsyJobInstanceService.count(Wrappers.<CsDsyJobInstance>lambdaQuery().eq(CsDsyJobInstance::getJobId, jobId)
                .in(CsDsyJobInstance::getInstanceStatus, InstanceStatus.RUNNING, InstanceStatus.WAITING)) > 0) {
            return R.fail("该任务有实例正在运行或待运行，中断后才能下线。");
        }
        CsDsyJob job = dsyJobService.getById(jobId);
        if (job == null) {
            return R.fail("任务流程不存在");
        }
        CsDsyJob entity = new CsDsyJob();
        entity.setId(jobId);
        entity.setJobStatus(jobStatus);
        if (jobStatus == CsJobStatus.OFFLINE && job.getTriggerStatus() == 1) { //任务需要下线，但是调度还在开启状态，需要关闭
            entity.setTriggerStatus(0);
            //需要关闭xxljob
            if (job.getXxlJobId() != null && job.getXxlJobId() > 0) {
                ReturnT<String> jobStop = adminBiz.jobStop(job.getXxlJobId());
                log.info("DSY任务:暂停任务|jobId={},xxljobId={},ret={}", jobId, job.getXxlJobId(), jobStop);
            }
        }
        entity.setUpdateBy(AuthUtil.getNickName());
        entity.setUpdateUserId(AuthUtil.getUserId());
        entity.setUpdateTime(LocalDateTime.now());
        return R.status(dsyJobService.updateById(entity));
    }

    /**
     *
     * @param job
     * @return
     */
    @ApiLog("数据同步任务")
    @PostMapping("/submitcron")
    @ApiOperation(value = "新增修改任务调度", httpMethod = "POST")
    public R<String> submitCron(@RequestBody CsDsyJob job) {
        if (job.getId() == null || StrUtil.isBlank(job.getJobCron())) {
            return R.fail("请求参数不正确");
        }
        Integer jobId = job.getId();
        CsDsyJob dbJob = dsyJobService.getById(jobId);
        if (job.getTriggerStatus() == 1 && dbJob.getJobStatus() == CsJobStatus.OFFLINE) {
            return R.fail("任务必须上线后才能开启任务调度");
        }
        dbJob.setJobCron(job.getJobCron());
        int xxlJobId = Optional.<Integer>ofNullable(dbJob.getXxlJobId()).orElse(0);
        int triggerStatus = Optional.<Integer>ofNullable(job.getTriggerStatus()).orElse(0);
        ReturnT<String> returnT = _handleJob(dbJob, AuthUtil.getUserId(), xxlJobId, triggerStatus);
        log.info("DSY任务|新增/修改xxljob任务|jobId={},symbol={},result={}", jobId, (xxlJobId > 0 ? "update" : "add"), returnT);
        if (ReturnT.SUCCESS_CODE == returnT.getCode()) {
            //首次添加任务，需要把任务ID更新进去
            CsDsyJob entity = new CsDsyJob();
            //修改的时候content=null，新增时候content=xxljobId
            int newXxlJobId = xxlJobId > 0 ? xxlJobId : Integer.valueOf(returnT.getContent());
            entity.setXxlJobId(newXxlJobId);
            entity.setTriggerStatus(triggerStatus);
            entity.setId(jobId);
            entity.setJobCron(job.getJobCron());
            dsyJobService.updateById(entity);
            //修改任务，并且之前为暂停状态：需要启动任务
            if (xxlJobId > 0 && triggerStatus == 1 && dbJob.getTriggerStatus() == 0) {
                ReturnT<String> jobStart = adminBiz.jobStart(newXxlJobId);
                log.info("DSY任务:启动任务|jobId={},xxljobId={},ret={}", jobId, newXxlJobId, jobStart);
            }
            if (xxlJobId > 0 && triggerStatus == 0 && dbJob.getTriggerStatus() == 1) {
                ReturnT<String> jobStop = adminBiz.jobStop(newXxlJobId);
                log.info("DSY任务:暂停任务|jobId={},xxljobId={},ret={}", jobId, newXxlJobId, jobStop);
            }
        }
        return R.data("操作成功");
    }

    /**
     * 处理xxljob任务
     * @return
     */
    private ReturnT<String> _handleJob(CsDsyJob jobInfo, Long userId, int jobId, int triggerStatus) {
        try {//校验cron的正确性
            PatternParser.parse(jobInfo.getJobCron());
        } catch (Exception e) {
            throw new ServiceException("请检查Cron表达式不正确");
        }
        JobInfo xxlJobInfo = new JobInfo();
        xxlJobInfo.setJobGroup(groupId);
        xxlJobInfo.setJobDesc("CsDSY任务:" + jobInfo.getJobName() + ":" + jobInfo.getId());
        xxlJobInfo.setAuthor(userId.toString());
        xxlJobInfo.setScheduleType("CRON"); //调度类型：固定速度【以固定速度，触发任务调度；按照固定的间隔时间，周期性触发】
        xxlJobInfo.setScheduleConf(jobInfo.getJobCron());
        xxlJobInfo.setMisfireStrategy("DO_NOTHING");  //调度过期策略：忽略【调度过期后，忽略过期的任务，从当前时间开始重新计算下次触发时间】
        xxlJobInfo.setExecutorRouteStrategy("ROUND"); //路由策略：轮询
        xxlJobInfo.setExecutorBlockStrategy("SERIAL_EXECUTION"); //阻塞处理策略：单机串行【调度请求进入单机执行器后，调度请求进入FIFO队列并以串行方式运行】
        xxlJobInfo.setExecutorHandler("dsyJobHandler");
        xxlJobInfo.setExecutorParam(jobInfo.getId().toString());
        xxlJobInfo.setExecutorTimeout(60); //超时时间1分钟
        xxlJobInfo.setExecutorFailRetryCount(0);
        xxlJobInfo.setGlueUpdatetime(new Date());
        xxlJobInfo.setGlueType("BEAN");
        xxlJobInfo.setTriggerLastTime(0);
        xxlJobInfo.setTriggerNextTime(0);
        xxlJobInfo.setTriggerStatus(triggerStatus);//1.运行 0.暂停
        if (jobId == 0) { //新增
            xxlJobInfo.setAddTime(new Date());
            return adminBiz.jobAdd(xxlJobInfo);
        } else {
            xxlJobInfo.setId(jobId);
            return adminBiz.jobUpdate(xxlJobInfo);
        }
    }
}