/**
 * 
 */
package org.springblade.csetl.controller;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.log.annotation.ApiLog;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.csetl.entity.CsDsyJobInstance;
import org.springblade.csetl.enums.CsInstanceStatus;
import org.springblade.csetl.service.CsDsyJobInstanceService;
import org.springblade.csetl.service.CsDsyJobService;
import org.springblade.system.client.DataPermClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 *	数据同步任务实例 操作处理
 *
 * @author aokunsang
 * @date   2023-09-21
 */
@RestController
@RequestMapping("/cs/dsy-job-instance")
@Api(value = "数据同步任务实例相关接口", tags = "数据同步任务实例接口")
public class CsDsyJobInstanceController extends BladeController {

	@Autowired
	private CsDsyJobInstanceService dsyJobInstanceService;
	
	@Autowired
	private CsDsyJobService dsyJobService;
	
	@Autowired
	private DataPermClient dataPermClient;
	
	/**
	 * 	列表数据
	 * @param dsyJobInstance
	 * @return
	 */
	@GetMapping("/merge-page")
	@ApiOperation(value = "分页查询任务分组聚合实例", httpMethod = "GET")
	public R<IPage<CsDsyJobInstance>> mergePageList(CsDsyJobInstance dsyJobInstance, Query query){
		LambdaQueryWrapper<CsDsyJobInstance> queryWrapper = Wrappers.<CsDsyJobInstance>lambdaQuery().orderByDesc(CsDsyJobInstance::getId);
		if(dsyJobInstance.getStartTime() != null && dsyJobInstance.getEndTime() != null) {
			queryWrapper
			.gt(CsDsyJobInstance::getCreateTime, DateUtil.format(dsyJobInstance.getStartTime(), "yyyy-MM-dd"))
			.lt(CsDsyJobInstance::getCreateTime, DateUtil.format(dsyJobInstance.getEndTime(), "yyyy-MM-dd"));
		}
		queryWrapper.eq(CsDsyJobInstance::getTenantId, AuthUtil.getTenantId())
					.eq(dsyJobInstance.getJobId() != null, CsDsyJobInstance::getJobId, dsyJobInstance.getJobId())
					.eq(dsyJobInstance.getInstanceRunType() != null, CsDsyJobInstance::getInstanceRunType, dsyJobInstance.getInstanceRunType());
		
		Long userId = getUser().getUserId();
		Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(userId);
		if(checkSuperPermRet == null || !checkSuperPermRet) {
			List<Integer> jobIdList = dsyJobService.jobIdListOfPerm(userId);
			if(CollectionUtil.isEmpty(jobIdList)) {
				return R.data(new Page<CsDsyJobInstance>());
			}
			queryWrapper.in(CsDsyJobInstance::getJobId, jobIdList);
		}
		LambdaQueryWrapper<CsDsyJobInstance> _queryWrapper = queryWrapper.clone();
		IPage<CsDsyJobInstance> instancePage = dsyJobInstanceService.customPage(((Page)Condition.getPage(query)).setSearchCount(false), queryWrapper.groupBy(CsDsyJobInstance::getJobId));
		return R.data(instancePage);
	}
	
	
	/**
	 * 	列表数据
	 * @param dsyJobInstance
	 * @return
	 */
	@GetMapping("/page")
	@ApiOperation(value = "分页根据任务ID查询任务实例", httpMethod = "GET")
	public R<IPage<CsDsyJobInstance>> pageList(CsDsyJobInstance dsyJobInstance, Query query){
		LambdaQueryWrapper<CsDsyJobInstance> queryWrapper = Condition.getQueryWrapper(dsyJobInstance).lambda().orderByDesc(CsDsyJobInstance::getId);
		if(dsyJobInstance.getStartTime() != null && dsyJobInstance.getEndTime() != null) {
			queryWrapper
			.gt(CsDsyJobInstance::getCreateTime, DateUtil.format(dsyJobInstance.getStartTime(), "yyyy-MM-dd"))
			.lt(CsDsyJobInstance::getCreateTime, DateUtil.format(dsyJobInstance.getEndTime(), "yyyy-MM-dd"));
		}
		Long userId = getUser().getUserId();
		Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(userId);
		if(checkSuperPermRet == null || !checkSuperPermRet) {
			List<Integer> jobIdList = dsyJobService.jobIdListOfPerm(userId);
			if(CollectionUtil.isEmpty(jobIdList)) {
				return R.data(new Page<CsDsyJobInstance>());
			}
			queryWrapper.in(CsDsyJobInstance::getJobId, jobIdList);
		}
		IPage<CsDsyJobInstance> instancePage = dsyJobInstanceService.page(Condition.getPage(query), queryWrapper);
//		if(instancePage != null && CollectionUtil.isNotEmpty(instancePage.getRecords())) {
//			instancePage.getRecords().forEach(item -> {
//				CsDsyJob job = dsyJobService.getById(item.getJobId());
//				item.setJobName(job == null ? "[已删除]" : job.getJobName());
//			});
//		}
		return R.data(instancePage);
	}
	
	/**
	 * 单条数据查询接口
	 * @param id
	 * @return
	 */
	@GetMapping("/info")
	@ApiOperation(value = "根据ID获取数据同步任务实例", httpMethod = "GET")
	public R<CsDsyJobInstance> info(@RequestParam Integer id) {
		return R.data(dsyJobInstanceService.getById(id));
	}
	
	/**
	 * 	删除数据接口
	 * @param id
	 * @return
	 */
	@ApiLog("数据同步任务实例")
	@PostMapping("/delete")
	@ApiOperation(value = "根据ID删除数据同步任务实例", httpMethod = "POST")
	public R<String> doDelete(@RequestParam Integer id) {
		CsDsyJobInstance dbJobInstance = dsyJobInstanceService.getById(id);
		if(dbJobInstance.getInstanceStatus() == CsInstanceStatus.RUNNING) {
			return R.fail("实例运行中，不能删除");
		}
		return R.status(dsyJobInstanceService.removeById(id));
	}
	
	/**
	 * 
	 * @param jobInstanceId
	 * @return
	 * @throws Exception 
	 */
	@ApiLog("数据同步任务实例")
	@PostMapping("/cancel")
	@ApiOperation(value = "根据ID取消/暂停任务实例", httpMethod = "POST")
	public R<String> doCancel(Integer jobInstanceId) throws Exception {
		dsyJobInstanceService.cancelJobInstance(jobInstanceId);
		return R.success("任务已取消");
	}
}