/**
 * 
 */
package org.springblade.csetl.controller;

import com.alibaba.fastjson2.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.log.annotation.ApiLog;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.csetl.dto.CsJobPermissionDto;
import org.springblade.csetl.service.CsDsyJobPermissionService;
import org.springblade.etl.dto.JobPermissionDoo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 *	数据同步任务权限组 操作处理
 *
 * @author aokunsang
 * @date   2023-09-21
 */
@RestController
@RequestMapping("/cs/dsy-job/permission")
@Api(value = "数据同步任务权限组相关接口", tags = "数据同步任务权限组接口")
public class CsDsyJobPermissionController extends BladeController {

	@Autowired
	private CsDsyJobPermissionService dsyJobPermissionService;
	
	/**
	 * 
	 * @param dataPermId
	 * @param jobCategoryId
	 * @return
	 */
	@GetMapping("/jobIdList")
	@ApiOperation(value = "根据权限组ID和分类ID查询任务流程ID列表")
	public R<List<Integer>> jobIdList(@RequestParam Integer dataPermId, @RequestParam Integer jobCategoryId){
		return R.data(dsyJobPermissionService.listJobId(dataPermId, jobCategoryId));
	}
	
	/**
	 * 
	 * @param jobId
	 * @return
	 */
	@GetMapping("/dataPermList/{jobId}")
	@ApiOperation(value = "根据数据同步任务ID查询权限组列表")
	public R<List<JSONObject>> dataPermList(@PathVariable Integer jobId){
		return R.data(dsyJobPermissionService.listDataPerm(jobId));
	}
	
	/**
	 *	修改数据接口
	 * @param jobDataPermDto
	 * @return
	 */
	@ApiLog("数据同步权限组")
	@PostMapping("/submit")
	@ApiOperation(value = "增加或修改数据同步任务权限组", httpMethod = "POST")
	public R<String> doUpdate(@RequestBody CsJobPermissionDto jobDataPermDto) {
		return R.status(dsyJobPermissionService.doSaveOrUpdate(jobDataPermDto, getUser()));
	}
	
	/**
	 * 任务流程列表的权限组管理入口
	 * @return
	 **/
	@ApiLog("数据同步权限组")
	@PostMapping("/perform")
	@ApiOperation(value = "批量添加或修改权限组与任务关系", notes = "任务列表-权限组管理入口【超级管理员才能操作】")
	public R<String> doPerform(@RequestBody JobPermissionDoo jobPermDoo) {
		if(!AuthUtil.isAdmin()) {
			return R.fail("只能管理员操作");
		}
		return R.status(dsyJobPermissionService.doPerform(jobPermDoo, getUser()));
	}
	
}