/*
 * Copyright (c) 2018-2028, AoKunSang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.csetl.engine;

import cn.hutool.core.map.MapUtil;
import com.alibaba.druid.util.JdbcUtils;
import lombok.extern.slf4j.Slf4j;
import org.springblade.csetl.dto.BaseQueryParam;
import org.springblade.csetl.dto.EsQueryParam;
import org.springblade.csetl.dto.MysqlQueryParam;
import org.springframework.stereotype.Component;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Map;
import java.util.Properties;

/**
 * mysql执行器
 *
 * @author xipengqiang
 * @datetime 2025-02-10 15:16:00
 */
@Slf4j
@Component
public class MysqlExecutor implements SqlExecutor<String> {

    @Override
    public Boolean check(BaseQueryParam queryParam) {
        MysqlQueryParam mysqlQueryParam = (MysqlQueryParam) queryParam;
        Connection connection = null;
        Statement statement = null;
        try {
            connection = createConnection(mysqlQueryParam);
            statement = connection.createStatement();
            statement.executeQuery("select 1");
            return true;
        } catch (Exception e) {
            log.error(e.getMessage());
        } finally {
            JdbcUtils.close(connection);
            JdbcUtils.close(statement);
        }
        return false;
    }

    /**
     * @param mysqlQueryParam
     * @return
     * @throws Exception
     */
    public Connection createConnection(MysqlQueryParam mysqlQueryParam) throws Exception {
        return createConnection(mysqlQueryParam, false);
    }

    /**
     * 创建连接
     *
     * @param mysqlQueryParam
     * @return
     * @throws SQLException
     */
    public Connection createConnection(MysqlQueryParam mysqlQueryParam, boolean heavyCall) throws Exception {
        Properties props = new Properties();
        props.put("user", mysqlQueryParam.getUsername());
        props.put("password", mysqlQueryParam.getPassword());
        //用于显示字段注释
        props.put("remarks", "true");
//		props.put("useInformationSchema", "true");
        props.put("remarksReporting", "true");
        props.put("nullCatalogMeansCurrent", "true");
        //获取每个数据库的超时时间
        Map<String, String> timeoutMap = jdbcConnectionPropTimeout(heavyCall);
        props.putAll(timeoutMap);
        return DriverManager.getConnection(mysqlQueryParam.getUrl(), props);
    }


    public Map<String, String> jdbcConnectionPropTimeout(boolean heavyCall) {
        //https://dev.mysql.com/doc/connector-j/8.0/en/connector-j-reference-configuration-properties.html
        String socketTimeout = heavyCall ? String.valueOf(60000) : String.valueOf(10000);
        return MapUtil.builder("connectTimeout", String.valueOf(2000))
                .put("socketTimeout", socketTimeout)
                .map();
    }


    @Override
    public String query(BaseQueryParam queryParam) {
        return "";
    }


    /**
     * 执行mysql语句
     * @return
     */
    public boolean execute() {
        return false;
    }
}
