package org.springblade.csetl.entity;

import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springblade.core.mp.base.TenantEntity;
import org.springblade.csetl.enums.CsDsTypeEnum;
import org.springblade.csetl.enums.CsJobStatus;

import java.time.LocalDateTime;

/**
 * 定制的数据同步任务表 cs_dsy_job
 *
 * @author yourname
 * @date 2023-09-21
 */
@Data
@TableName("cs_dsy_job")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "CsDsyJob对象", description = "定制的数据同步任务")
public class CsDsyJob extends TenantEntity {

    private static final long serialVersionUID = 1L;

    /**
     * 主键ID
     */
    @TableId(type = IdType.AUTO)
    private Integer id;

    /**
     * 租户ID
     */
    @ApiModelProperty(value = "租户ID")
    private String tenantId;

    /**
     * 分类ID
     */
    @ApiModelProperty(value = "分类ID")
    private Integer categoryId;

    /**
     * 任务名称
     */
    @ApiModelProperty(value = "任务名称")
    @TableField(condition = SqlCondition.LIKE)
    private String jobName;

    /**
     * 任务描述
     */
    @ApiModelProperty(value = "任务描述")
    private String jobDesc;

    /**
     * 上线状态
     */
    @ApiModelProperty(value = "上线状态")
    private CsJobStatus jobStatus;

    /**
     * 调度状态
     */
    @ApiModelProperty(value = "调度状态(0-下线/1-上线)")
    private Integer triggerStatus;

    /**
     * xxljob的任务Cron
     */
    @ApiModelProperty(value = "xxljob的任务Cron")
    private String jobCron;

    /**
     * xxljob的任务ID
     */
    @ApiModelProperty(value = "xxljob的任务ID")
    private Integer xxlJobId;

    /**
     * 输入数据源ID
     */
    @ApiModelProperty(value = "输入数据源ID")
    private Integer sourceDsId;

    /**
     * 输入数据源类型
     */
    @ApiModelProperty(value = "输入数据源类型")
    private CsDsTypeEnum sourceDsType;

    /**
     * 输入数据源表名
     */
    @ApiModelProperty(value = "输入数据源表名")
    private String sourceTableName;

    /**
     * 目标数据源ID
     */
    @ApiModelProperty(value = "目标数据源ID")
    private Integer targetDsId;

    /**
     * 目标数据源类型
     */
    @ApiModelProperty(value = "目标数据源类型")
    private CsDsTypeEnum targetDsType;

    /**
     * 目标数据源表名
     */
    @ApiModelProperty(value = "目标数据源表名")
    private String targetTableName;

    /**
     * 输入数据源的查询SQL
     */
    @ApiModelProperty(value = "输入数据源的查询SQL")
    private String querySql;

    /**
     * 字段映射json数组
     */
    @ApiModelProperty(value = "字段映射json数组")
    private String columnJson;


    /**
     * 公开状态(0-不公开/1-公开)
     */
    @ApiModelProperty(value = "公开状态(0-不公开/1-公开)")
    private Integer publicStatus;

    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    private LocalDateTime createTime;

    /**
     * 创建人ID
     */
    @ApiModelProperty(value = "创建人ID")
    private Long createUserId;

    /**
     * 创建人
     */
    @ApiModelProperty(value = "创建人")
    private String createBy;

    /**
     * 更新时间
     */
    @ApiModelProperty(value = "更新时间")
    private LocalDateTime updateTime;

    /**
     * 更新人ID
     */
    @ApiModelProperty(value = "更新人ID")
    private Long updateUserId;

    /**
     * 更新人
     */
    @ApiModelProperty(value = "更新人")
    private String updateBy;
}
