/**
 * 
 */
package org.springblade.csetl.mapper;


import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springblade.csetl.dto.CsInstanceCountVO;
import org.springblade.csetl.entity.CsDsyJobInstance;

import java.util.List;

/**
 *	数据同步任务实例 数据处理
 *
 * @author aokunsang
 * @date   2023-09-21
 */
public interface CsDsyJobInstanceMapper extends BaseMapper<CsDsyJobInstance> {


	@InterceptorIgnore(tenantLine = "true")
	@Select("select * from cs_dsy_job_instance where instance_status = 1")
	List<CsDsyJobInstance> selectAllRunningInstanceList();
	
	@InterceptorIgnore(tenantLine = "true")
	@Select("select * from cs_dsy_job_instance where instance_status = 0")
	List<CsDsyJobInstance> selectAllWaitingInstanceList();

	@InterceptorIgnore(tenantLine = "true")
	@Select("select job_id, min(start_time) as start_time, max(end_time) as end_time, max(id) as id, sum(timestampdiff(SECOND, start_time, end_time)) as total_secs from cs_dsy_job_instance ${ew.customSqlSegment}")
	public IPage<CsDsyJobInstance> customPage(IPage<CsDsyJobInstance> page, @Param(Constants.WRAPPER) LambdaQueryWrapper<CsDsyJobInstance> queryWrapper);

	@InterceptorIgnore(tenantLine = "true")
	@Select("select count(distinct job_id) from cs_dsy_job_instance ${ew.customSqlSegment}")
	public Long countPage(@Param(Constants.WRAPPER) LambdaQueryWrapper<CsDsyJobInstance> queryWrapper);


	@Select("select count(1)                                             as total,\n" +
			"       SUM(CASE WHEN instance_status = 1 THEN 1 ELSE 0 END) as runningCount,\n" +
			"       SUM(CASE WHEN instance_status = 2 THEN 1 ELSE 0 END)    successCount,\n" +
			"       SUM(CASE WHEN instance_status = 3 THEN 1 ELSE 0 END)    failCount,\n" +
			"       SUM(CASE WHEN instance_status = 4 THEN 1 ELSE 0 END)    cancelCount,\n" +
			"       SUM(CASE WHEN instance_status = 5 THEN 1 ELSE 0 END)    waitCount\n" +
			"from cs_dsy_job_instance")
    CsInstanceCountVO getCounts();

	@Select("select count(1)                             as              total,\n" +
			"       DATE_FORMAT(create_time, '%Y-%m-%d') as              date,\n" +
			"       SUM(CASE WHEN instance_status = 2 THEN 1 ELSE 0 END) successCount,\n" +
			"       SUM(CASE WHEN instance_status = 3 THEN 1 ELSE 0 END) failCount,\n" +
			"       SUM(CASE WHEN instance_status = 5 THEN 1 ELSE 0 END) waitCount\n" +
			"FROM cs_dsy_job_instance\n" +
			"where DATE_SUB(CURDATE(), INTERVAL #{day} DAY) <= date(create_time)\n" +
			"group by DATE_FORMAT(create_time, '%Y-%m-%d')\n" +
			"order by DATE_FORMAT(create_time, '%Y-%m-%d')")
	List<CsInstanceCountVO> getCountByDays(int day);

	@Select("select count(1)                             as              total,\n" +
			"       DATE_FORMAT(create_time, '%Y-%m-%d') as              `date`,\n" +
			"       SUM(CASE WHEN instance_status = 1 THEN 1 ELSE 0 END) runningCnt,\n" +
			"       SUM(CASE WHEN instance_status = 2 THEN 1 ELSE 0 END) successCount,\n" +
			"       SUM(CASE WHEN instance_status = 3 THEN 1 ELSE 0 END) failCount,\n" +
			"       SUM(CASE WHEN instance_status = 4 THEN 1 ELSE 0 END) cancelCount,\n" +
			"       SUM(CASE WHEN instance_status = 5 THEN 1 ELSE 0 END) waitCount\n" +
			"FROM cs_dsy_job_instance\n" +
			"where DATE_FORMAT(create_time, 'y%Y-%m-%d') <= DATE_FORMAT(now(), '%Y-%m-%d')\n" +
			"group by `date`")
	CsInstanceCountVO getCountToday();

	@Select("select count(1)                                             as total,\n" +
			"       DATE_FORMAT(create_time, '%Y-%m-%d %H')              as date,\n" +
			"       SUM(CASE WHEN instance_status = 1 THEN 1 ELSE 0 END) as runningCount,\n" +
			"       SUM(CASE WHEN instance_status = 2 THEN 1 ELSE 0 END)    successCount,\n" +
			"       SUM(CASE WHEN instance_status = 3 THEN 1 ELSE 0 END)    failCount,\n" +
			"       SUM(CASE WHEN instance_status = 4 THEN 1 ELSE 0 END)    cancelCount\n" +
			"FROM cs_dsy_job_instance\n" +
			"where DATE(create_time) = CURDATE()\n" +
			"group by DATE_FORMAT(create_time, '%Y-%m-%d %H')")
	List<CsInstanceCountVO> instanceLineChart();

	@Select("select dj.job_name jobName, dji.*\n" +
			"from cs_dsy_job_instance dji\n" +
			"         inner join cs_dsy_job dj on dji.job_id = dj.id\n" +
			"order by dji.start_time desc")
    IPage<CsDsyJobInstance> pageDo(IPage<CsDsyJobInstance> page);
}