/**
 * 
 */
package org.springblade.csetl.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.log.config.ServerInfo;
import org.springblade.csetl.dto.CsInstanceCountVO;
import org.springblade.csetl.entity.CsDsyJobInstance;
import org.springblade.csetl.enums.CsInstanceRunType;
import org.springblade.csetl.mapper.CsDsyJobInstanceMapper;
import org.springblade.datasource.context.DialectContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 *	数据同步任务实例 服务处理
 *
 * @author aokunsang
 * @date   2023-09-21
 */
@Slf4j
@Service
public class CsDsyJobInstanceService extends ServiceImpl<CsDsyJobInstanceMapper, CsDsyJobInstance>{
	


	@Autowired
	private CsDsyJobService csDsyJobService;
	
	@Autowired
	private CsReportDatasourceService datasourceService;

	@Autowired
	private CsQueryWrapperService queryWrapperService;
	
	@Autowired
	private DialectContext dialectContext;

	/**
	 * 查询正在运行的所有任务实例
	 * @return
	 */
	public List<CsDsyJobInstance> listAllRunningInstanceList(){
		return baseMapper.selectAllRunningInstanceList();
	}
	
	/**
	 * 查询待运行的所有任务实例
	 * @return
	 */
	public List<CsDsyJobInstance> listAllWaitingInstanceList(){
		return baseMapper.selectAllWaitingInstanceList();
	}
	

	/**
	 * 
	 * @param page
	 * @param queryWrapper
	 * @return
	 */
	public IPage<CsDsyJobInstance> customPage(IPage<CsDsyJobInstance> page, LambdaQueryWrapper<CsDsyJobInstance> queryWrapper){
		return baseMapper.customPage(page, queryWrapper);
	}
	
	/**
	 * 
	 * @param queryWrapper
	 * @return
	 */
	public Long countPage(LambdaQueryWrapper<CsDsyJobInstance> queryWrapper) {
		return baseMapper.countPage(queryWrapper);
	}
	

	/**
	 * 取消任务实例
	 * @param jobInstanceId
	 * @throws Exception 
	 */
	public void cancelJobInstance(Integer jobInstanceId) throws Exception {
//		CsDsyJobInstance jobInstance = getById(jobInstanceId);
//		if(jobInstance.getInstanceStatus() == CsInstanceStatus.RUNNING && !"unknown".equals(jobInstance.getServerIp())) {
//			String errorMsg = dataxJobService.cancel(jobInstance.getId(), jobInstance.getDataxConfigName(), jobInstance.getServerIp());
//			if(StrUtil.isNotEmpty(errorMsg)) {
//				throw new ServiceException("取消任务实例失败，失败原因：" + errorMsg);
//			}
//		}
//		CsDsyJobInstance updateEntity = new CsDsyJobInstance();
//		updateEntity.setId(jobInstanceId);
//		updateEntity.setEndTime(new Date());
//		updateEntity.setInstanceStatus(CsInstanceStatus);
//		updateById(updateEntity);
	}
	
	/**
	 * 刷新数据同步任务实例
	 * @param jobInstanceId
	 */
	public void refreshJobInstance(Integer jobInstanceId) {
//		CsDsyJobInstance dsyJobInstance = getById(jobInstanceId);
//		if(dsyJobInstance.getInstanceStatus() != CsInstanceStatus.RUNNING) {
//			return;
//		}
//
//		CsDsyJobInstance updateEntity = new CsDsyJobInstance();
//		updateEntity.setId(dsyJobInstance.getId());
//
//		//读取datax的log日志信息，并且初始化到updateEntity
//		dataxJobService.readMetrics(updateEntity, dsyJobInstance.getServerIp(), dsyJobInstance.getDataxLogName());
//
//		//获取当前datax作业执行状态
//		CsInstanceStatus instanceStatus = updateEntity.getInstanceStatus();
//
//		//任务已结束
//		boolean isDone = instanceStatus == CsInstanceStatus.SUCCESS || instanceStatus == CsInstanceStatus.FAILURE;
//		if(isDone && updateEntity.getEndTime() == null) {
//			updateEntity.setEndTime(new Date());
//		}
//
//		//更新状态数据
//		updateById(updateEntity);
//
//		//如果开启增量，任务同步成功后，需要更新最大值和最小值到jobTable
//		if(instanceStatus == InstanceStatus.SUCCESS && dsyJobInstance.getSyncType() == DataSyncType.INCREMENT) {
//			DsyJobTable tableEntity = new DsyJobTable();
//			tableEntity.setId(dsyJobInstance.getJobTableId());
//			tableEntity.setStartIncrementData(dsyJobInstance.getStartIncrementData());
//			tableEntity.setEndIncrementData(dsyJobInstance.getEndIncrementData());
//			dsyJobTableService.updateById(tableEntity);
//		}
//
//		//没有结束，扔到线程池中继续轮询
//		if(!isDone) {
//			DaemonFactory.addTask(new DaemonTaskConfig(jobInstanceId.toString(), DaemonTaskType.datax_job));
//		}else { //已结束，发送消息开始执行下一个任务
//			SpringUtil.publishEvent(new JobRunEvent());
//		}
	}
	
	/**
	 * 运行实例
	 * @param jobInstanceId
	 * @param runType
	 */
	public void runJobInstance(Integer jobInstanceId, CsInstanceRunType runType, ServerInfo serverInfo) {
//		CsDsyJobInstance dsyJobInstance = getById(jobInstanceId);
//		if(dsyJobInstance.getInstanceStatus() != CsInstanceStatus.WAITING) {
//			return;
//		}
//		//更新状态为运行中
//		CsDsyJobInstance uptJobInstance = new CsDsyJobInstance();
//		uptJobInstance.setId(jobInstanceId);
//		uptJobInstance.setInstanceStatus(CsInstanceStatus.RUNNING);
//		updateById(uptJobInstance);
//
//		DataxRes dataxRes = null;
//		String errMsg = null;
//		Date startTime = null;
//		CsDataSyncType dataSyncType = null;
//		try {
//			CsDsyJob dsyJob = csDsyJobService.getById(dsyJobInstance.getJobId());
//			DsyJobTable jobTable = dsyJobTableService.getById(dsyJobInstance.getJobTableId());
//
//			Assert.notNull(dsyJob, "任务已删除");
//			Assert.notNull(jobTable, "任务表已删除");
//			dataSyncType = Optional.<CsDataSyncType>ofNullable(jobTable.getSyncType()).orElse(dsyJob.getSyncType());
//
//			//查询source和target数据源信息
//			List<CsReportDatasource> dsList = datasourceService.listByIds(Arrays.asList(dsyJob.getSourceDsId(), dsyJob.getTargetDsId()));
//			Map<Integer, CsReportDatasource> dsMap = dsList.stream().collect(Collectors.toMap(CsReportDatasource::getId, t -> t));
//			CsReportDatasource sourceDs = dsMap.get(dsyJob.getSourceDsId());
//			CsReportDatasource targetDs = dsMap.get(dsyJob.getTargetDsId());
//			Assert.notNull(sourceDs, "输入数据源不存在");
//			Assert.notNull(targetDs, "目标数据源不存在");
//
//			DataxReq dataxReq = new DataxReq();
//
//			dataxReq.setSyncType(dataSyncType);
//			dataxReq.setJobTableId(jobTable.getId());
//
//			String sourcePluginPrefix = sourceDs.getDsTypeSyl() == DsTypeEnum.dameng ? "rdbms" : sourceDs.getDsTypeSyl().name().toLowerCase();
//			String targetPluginPrefix = targetDs.getDsTypeSyl() == DsTypeEnum.dameng ? "rdbms" : targetDs.getDsTypeSyl().name().toLowerCase();
//
//			dataxReq.setSourcePlugin(sourcePluginPrefix + "reader");
//			dataxReq.setSourceJdbcUrl(sourceDs.getUrl().replaceAll("CONVERT_TO_NULL", "convertToNull"));
//			dataxReq.setSourceUser(sourceDs.getDsUser());
//			dataxReq.setSourcePassword(sourceDs.getDsPassword());
//			dataxReq.setQuerySql(jobTable.getQuerySql());
//
//			dataxReq.setPreSql(jobTable.getPreSql());
//			dataxReq.setTargetPlugin(targetPluginPrefix + "writer");
//			dataxReq.setTargetJdbcUrl(targetDs.getUrl().replaceAll("CONVERT_TO_NULL", "convertToNull"));
//			dataxReq.setTargetUser(targetDs.getDsUser());
//			dataxReq.setTargetPassword(targetDs.getDsPassword());
//			String targetTableName = dialectContext.getDialect(targetDs.getDsTypeSyl()).dbTableName(targetDs, jobTable.getTargetTableName());
//			dataxReq.setTargetTableName(targetTableName);
//
//			List<TargetTableColumnDto> targetColumnList = EtlJsonUtil.parseJsonArray(jobTable.getTargetColumnJson(), TargetTableColumnDto.class);
//			dataxReq.setTargetColumns(targetColumnList.stream().map(item -> "\"" + dialectContext.getDialect(targetDs.getDsTypeSyl()).dbEscape(item.getColumnName()) +  "\"").collect(Collectors.joining(",")));
//
//			//构造自动建表model
//			AutoCreateTableDto autoCreateTable = null;
//			//是否自动建表
//			if(Optional.<Integer>ofNullable(jobTable.getCreateTableStatus()).orElse(0) == 1) {
//				autoCreateTable = new AutoCreateTableDto();
//				autoCreateTable.setTableComment(jobTable.getTargetTableComment());
//				autoCreateTable.setJobTableId(jobTable.getId());
//				autoCreateTable.setSyncType(dataSyncType);
//				autoCreateTable.setTableName(jobTable.getTargetTableName());
//				autoCreateTable.setModelList(_buildCreateTableModel(targetColumnList));
//			}
//
//			//是否增量同步
//			if(dataSyncType == DataSyncType.INCREMENT) {
//				dataxReq.setStartIncrementData(jobTable.getEndIncrementData());
//				//查询最大值
//				dataxReq.setEndIncrementData(_queryMaxValue(sourceDs, jobTable.getIncrementColumnName(), jobTable.getSourceTableName()));
//				String whereSql = "";
//				if(StrUtil.isNotBlank(dataxReq.getEndIncrementData())) {
//					String incrConditionSql = incrementConditionSql(jobTable, dataxReq.getStartIncrementData(), dataxReq.getEndIncrementData());
//					whereSql = StrUtil.containsIgnoreCase(dataxReq.getQuerySql(), " where ") ? (" and " + incrConditionSql) : (" where " + incrConditionSql);
//				}
//				dataxReq.setQuerySql(dataxReq.getQuerySql() + whereSql);
//			}
//
//			//设置datax mysqlwriter的特殊扩展参数[Mysql支持upsert语义设置，且该参数仅支持mysql]
//			if(targetDs.getDsTypeSyl() == DsTypeEnum.mysql) {
//				dataxReq.setTargetCustomParamJson("\"writeMode\": \"update\",");
//			}
//
//			startTime = new Date();
//			//开始自动建表操作
//			if(autoCreateTable != null) {
//				long oneTime = System.currentTimeMillis();
//				ReportDatasource _sourceDs = new ReportDatasource();
//				_sourceDs.setDsType(DsTypeEnum.common.getValue());
//				transformContext.transformAndCreatTable(_sourceDs, targetDs,
//						autoCreateTable.getModelList(), null, autoCreateTable.getTableName(), autoCreateTable.getTableComment(), false);
//				log.info("数据同步:自动建表结束|cost:{}ms", (System.currentTimeMillis() - oneTime));
//			}
//
//			//非同步结构开始任务执行
//			if(dataSyncType != DataSyncType.STRUCTURE) {
//				dataxRes = dataxJobService.submit(dataxReq, serverInfo);
//			}
//		}catch (Exception e) {
//			log.error(e.getMessage(), e);
//			errMsg = LogTraceUtil.getStackTraceAsString(e);
//		} finally {
//			CsDsyJobInstance jobInstance = new CsDsyJobInstance();
//			jobInstance.setId(jobInstanceId);
//			jobInstance.setServerIp(serverInfo.getIp());
//			jobInstance.setInstanceType(InstanceType.BATCH);
//			jobInstance.setCreateTime(LocalDateTime.now());
//			jobInstance.setTenantId(dsyJobInstance.getTenantId());
//			jobInstance.setSyncType(dataSyncType);
//			jobInstance.setInstanceRunType(runType);
//			//任务包含两种结果形式：1.同步表结构；2.增量或全量数据同步
//			if(dataxRes != null) {
//				jobInstance.setStartTime(dataxRes.getStartTime());
//				jobInstance.setStartIncrementData(dataxRes.getStartIncrementData());
//				jobInstance.setEndIncrementData(dataxRes.getEndIncrementData());
//				jobInstance.setErrMsg(dataxRes.getErrorMsg());
//				jobInstance.setDataxConfigName(dataxRes.getConfigName());
//				jobInstance.setDataxLogName(dataxRes.getLogName());
//				jobInstance.setInstanceStatus(StrUtil.isEmpty(dataxRes.getErrorMsg()) ? InstanceStatus.RUNNING : InstanceStatus.FAILURE);
//			}else {
//				jobInstance.setStartTime(startTime);
//				jobInstance.setErrMsg(errMsg);
//				jobInstance.setInstanceStatus(StrUtil.isEmpty(errMsg) ? InstanceStatus.SUCCESS : InstanceStatus.FAILURE);
//			}
//			if(dataxRes == null || StrUtil.isNotEmpty(dataxRes.getErrorMsg())) {
//				jobInstance.setEndTime(new Date());
//			}
//			updateById(jobInstance);
//			//运行成功，添加到任务轮训调度中
//			if(dataxRes != null && StrUtil.isEmpty(dataxRes.getErrorMsg())) {
//				DaemonFactory.addTask(new DaemonTaskConfig(jobInstance.getId().toString(), DaemonTaskType.datax_job));
//			}
//		}
	}
	



    public CsInstanceCountVO getCounts() {

		return baseMapper.getCounts();
    }

	public List<CsInstanceCountVO> getCountByDays(int day) {

		return baseMapper.getCountByDays(day);
	}

	public CsInstanceCountVO getCountToday() {

		return baseMapper.getCountToday();
	}

	public List<CsInstanceCountVO> instanceLineChart() {

		return baseMapper.instanceLineChart();
	}

    public IPage<CsDsyJobInstance> pageDo(IPage<CsDsyJobInstance> page) {

		return baseMapper.pageDo(page);
    }
}