/**
 *
 */
package org.springblade.csetl.service;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.csetl.dto.BaseQueryParam;
import org.springblade.csetl.dto.EsQueryParam;
import org.springblade.csetl.dto.InfluxdbQueryParam;
import org.springblade.csetl.dto.MysqlQueryParam;
import org.springblade.csetl.engine.ElasticsearchExecutor;
import org.springblade.csetl.engine.InfluxdbExecutor;
import org.springblade.csetl.engine.MysqlExecutor;
import org.springblade.csetl.entity.CsReportDatasource;
import org.springblade.csetl.enums.CsDsTypeEnum;
import org.springblade.csetl.mapper.CsReportDatasourceMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author xipengqiang
 * @date 2025-02-07
 */
@Service
public class CsReportDatasourceService extends ServiceImpl<CsReportDatasourceMapper, CsReportDatasource> {

    @Autowired
    private ElasticsearchExecutor elasticsearchExecutor;

    @Autowired
    private InfluxdbExecutor influxdbExecutor;

    @Autowired
    private MysqlExecutor mysqlExecutor; ;

    public Boolean testDs(CsReportDatasource csReportDatasource) {

        BaseQueryParam baseQueryParam = new BaseQueryParam();
        baseQueryParam
                .setHost(csReportDatasource.getDsUrl())
                .setPort(csReportDatasource.getDsPort())
                .setPassword(csReportDatasource.getDsPassword())
                .setUsername(csReportDatasource.getDsName());
        CsDsTypeEnum dsType = csReportDatasource.getDsType();
        switch (dsType) {
            case MYSQL:
                MysqlQueryParam mysqlQueryParam = new MysqlQueryParam();
                BeanUtil.copyProperties(baseQueryParam, mysqlQueryParam);
                mysqlQueryParam.setUrl(csReportDatasource.getDsUrl());
                return mysqlExecutor.check(mysqlQueryParam);
            case ES:
                EsQueryParam esQueryParam = new EsQueryParam();
                BeanUtil.copyProperties(baseQueryParam, esQueryParam);
                esQueryParam.setPwd(csReportDatasource.getDsPassword());
                return elasticsearchExecutor.check(esQueryParam);
            case INFLUXDB:
                InfluxdbQueryParam influxdbQueryParam = new InfluxdbQueryParam();
                BeanUtil.copyProperties(baseQueryParam, influxdbQueryParam);
                influxdbQueryParam
                        .setFluxV2Status(influxdbQueryParam.getFluxV2Status())
                        .setDatabase(csReportDatasource.getDsDb())
                        .setToken(csReportDatasource.getFluxToken());
                return influxdbExecutor.check(influxdbQueryParam);
        }
        return false;
    }


}